@@ -130,8 +130,7 @@ complete_connection(NMDevice            *device,
                               NULL,
                               _("Team connection"),
                               "team",
-                              NULL,
-                              TRUE);
+                              NULL);
 
     _nm_connection_ensure_setting(connection, NM_TYPE_SETTING_TEAM);
 
